import bpy
import math
import random


#Constants
dimension      = {'X': 100,  #Length (mm)
                  'Y': 20,  #Width (mm)
                  'Z': 20}  #Height (mm)
gradientLength = 85         #Gradient length (mm)
barSize        = {'X': 1,   #Bar width (mm)
                  'Z': 0.8} #Bar height (mm)


#Methods

#Generate vertices
def getVerts(dimensionNumber, barSize):
    verts = []
    
    for x in range(0, dimensionNumber['X'] + 1):
        for z in range(0, dimensionNumber['Z'] + 1):
            verts.append( (x * barSize['X'], 0, z * barSize['Z']) )
            verts.append( (x * barSize['X'], dimensionNumber['Y'], z * barSize['Z']) )
    
    return verts
    
#Generate gradient points, assuming x as the gradient direction
def getGradient(dimensionNumber, gradientLength, barSize):
    gradientNumber = math.floor(gradientLength / barSize['X'])
    xMin = (dimensionNumber['X'] - gradientNumber) / 2
    xMax = (dimensionNumber['X'] + gradientNumber) / 2 
    gradient = []
    
    for x in range(0, dimensionNumber['X']):
        
        if (x > xMax):
            alpha = 1
        elif(x < xMin):
            alpha = 0
        else:
            alpha = repartitionFunction(x, xMin, 0, xMax, 1)
        
        xArray = []
        for z in range(0, dimensionNumber['Z']):
            if (random.random() < alpha):
                bar = True
            else:
                bar = False
            xArray.append(bar)
        gradient.append(xArray)
    
    return gradient

#Generate faces
def getFaces(dimensionNumber, gradient):
    faces = []
    xMax  = dimensionNumber['X']
    zMax  = dimensionNumber['Z']
        
    for x in range(0, xMax):
        for z in range(0, zMax):
            if (gradient[x][z]):
                n = 2 * z + x * (zMax + 1) * 2
                cube = [
                    n,
                    n + (zMax + 1) * 2,
                    n + (zMax + 1) * 2 + 1,
                    n + 1,
                    n + 2,
                    n + (zMax + 1) * 2 + 2,
                    n + (zMax + 1) * 2 + 3,
                    n + 3
                ]
                
                #yMin and yMax faces
                faces.append( (cube[0], cube[1], cube[5], cube[4]) )
                faces.append( (cube[2], cube[3], cube[7], cube[6]) )
                if (x == 0 or not gradient[x - 1][z]):
                    faces.append( (cube[0], cube[4], cube[7], cube[3]) )
                if (z == 0 or not gradient[x][z - 1]):
                    faces.append( (cube[0], cube[1], cube[2], cube[3]) )
                if (x == xMax - 1 or not gradient[x + 1][z]):
                    faces.append( (cube[1], cube[2], cube[6], cube[5]) )
                if (z == zMax - 1 or not gradient[x][z + 1]):
                    faces.append( (cube[4], cube[5], cube[6], cube[7]) ) 
                
                if (x == xMax - 1 and z == zMax - 1):
                    print('xMax:', xMax)
                    print('zMax:', zMax)
                    print('n:', n)
                    print('Cube 6:', cube[6])          
    return faces
    
#Make gradient dot object
def makeGradient(dimensionNumber, barSize, gradientLength, gradient, name):
    verts = getVerts(dimensionNumber, barSize)
    print('Verts length:', len(verts))  
    faces = getFaces(dimensionNumber, gradient)
    
    #Create mesh and object
    mesh = bpy.data.meshes.new(name)
    object = bpy.data.objects.new(name, mesh)
    
    #Set mesh location
    bpy.context.scene.cursor_location = ( 
        - dimensionNumber['X'] * barSize['X'] / 2, 
        - dimensionNumber['Y'] / 2, 
        0)
    object.location = bpy.context.scene.cursor_location
    bpy.context.scene.objects.link(object)
    
    #Create mesh from data
    mesh.from_pydata(verts, [], faces)
    mesh.update(calc_edges = True)

#Make gradient and its boolean negation
def makeDualGradient(dimension, barSize, gradientLength):

    dimensionNumber = {}
    dimensionNumber['X'] = math.floor(dimension['X'] / barSize['X'])
    dimensionNumber['Y'] = dimension['Y']
    dimensionNumber['Z'] = math.floor(dimension['Z'] / barSize['Z'])

    gradient = getGradient(dimensionNumber, gradientLength, barSize)
    makeGradient(dimensionNumber, barSize, gradientLength, gradient, 'Extruder 0')
    for x in range(0, dimensionNumber['X']):
        for z in range(0, dimensionNumber['Z']):
                gradient[x][z] = not gradient[x][z]
    makeGradient(dimensionNumber, barSize, gradientLength, gradient, 'Extruder 1')

#Delete all objects from the scene
def deleteAll():
    selectAll() 
    bpy.ops.object.delete()
    for item in bpy.data.meshes:
      bpy.data.meshes.remove(item)
    return True

#Select all objects in the scene
def selectAll():
    bpy.ops.object.select_all(action='SELECT')
    return True 
    
#Linear repartition function
def repartitionFunction(x, x0, y0, x1, y1):
    return (y1 - y0) / (x1 - x0) * (x - x0) + y0
    
    
#Script    
deleteAll()
makeDualGradient(dimension, barSize, gradientLength)
