import bpy
import math

#Constants

scale = 1/10

nozzleSize = 0.5 * scale
base       = 54 * scale
height     = 70 * scale

layerHeight    = nozzleSize * 0.8
edgeThickness  = nozzleSize * 2
gridThickness  = nozzleSize
minNumber      = 1
maxNumber      = 20


#Functions

def layerDimension(base, numberOfLayers, layerIndex, minNumber, maxNumber, gridThickness):
    numberOfBars = repartitionFunction(layerIndex, 0, maxNumber, numberOfLayers, minNumber)
    gridSpacing = (base - numberOfBars * gridThickness) / (numberOfBars + 1)
    return {'gridSpacing': gridSpacing, 'numberOfBars': numberOfBars}

def repartitionFunction(x, x0, y0, x1, y1):
    if x1 == x0:
        print('math error')
        return False
    else:
        return math.floor((y1 - y0) / (x1 - x0) * (x - x0) + y0)  

def makeCube(location):
    bpy.ops.mesh.primitive_cube_add(radius = 0.5, location = location)
    return bpy.context.object

def makeCuboid(location, dimension):
    cuboid = makeCube(location)
    cuboid.scale = dimension
    return cuboid

def makeLayer(base, numberOfLayers, layerIndex, layerHeight, minNumber, maxNumber, gridThickness):
    thisLayerDimension = layerDimension(base, numberOfLayers, layerIndex, minNumber, maxNumber, gridThickness)
    gridSpacing = thisLayerDimension['gridSpacing']
    numberOfBars = thisLayerDimension['numberOfBars']
    location = [0, 0, layerHeight * (layerIndex + 0.5)]
    
    if layerIndex % 2 == 0:
        dimension = (gridThickness, base, layerHeight)
        for barIndex in range(0, numberOfBars):
            location[0] = -base / 2 + (barIndex + 1) * gridSpacing + (barIndex + 0.5) * gridThickness 
            makeCuboid(tuple(location), dimension)    
    else:
        dimension = (base, gridThickness, layerHeight)
        for barIndex in range(0, numberOfBars):
            location[1] = -base / 2 + (barIndex + 1) * gridSpacing + (barIndex + 0.5) * gridThickness 
            makeCuboid(tuple(location), dimension)
    return True  

def makeGrid(base, height, layerHeight, minNumber, maxNumber, gridThickness):
    numberOfLayers = math.floor(height / layerHeight)  
    for layerIndex in range(0, numberOfLayers):
        makeLayer(base, numberOfLayers, layerIndex, layerHeight, minNumber, maxNumber, gridThickness)
    return True

def selectAll():
    bpy.ops.object.select_all(action='SELECT')
    return True 
    
def deleteAll():
    selectAll() 
    bpy.ops.object.delete()
    for item in bpy.data.meshes:
      bpy.data.meshes.remove(item)
    return True
      
def mergeAll(name = 'undefined' , centerLocation = (0, 0, 0)):
    selectAll()
    bpy.ops.object.join()
    bpy.context.scene.cursor_location = centerLocation
    bpy.ops.object.origin_set(type = 'ORIGIN_CURSOR') 
    renameSelected(name)
    return bpy.ops.object  
    
def renameSelected(name):
    object = bpy.context.selected_objects[0]
    object.name = name
    return object
    
def makeEdges(base, height, edgeThickness):
    location = (0, 0, height/2)
    outerDimension = (base, base, height)
    innerDimension = (base - edgeThickness * 2, base - edgeThickness * 2, height)
    positiveObject = makeCuboid(location, outerDimension)    
    negativeObject = makeCuboid(location, innerDimension)
    booleanSubstract(positiveObject, negativeObject)
    return True

def booleanSubstract(positiveObject, negativeObject):  
   bpy.context.scene.objects.active = positiveObject
   bpy.ops.object.modifier_add(type='BOOLEAN') 
   modifier = positiveObject.modifiers 
   modifier[0].name = "booleanSubstract" 
   modifier[0].object = negativeObject 
   modifier[0].operation = 'DIFFERENCE' 
   bpy.ops.object.modifier_apply(apply_as='DATA', modifier=modifier[0].name)
   bpy.context.scene.objects.active = negativeObject
   bpy.ops.object.delete()
   return True


    
# Script
    
deleteAll()
makeEdges(base, height, edgeThickness)
makeGrid(base - edgeThickness*2, height, layerHeight, minNumber, maxNumber, gridThickness)
mergeAll('gradientGrid')
print('Grid generated')